#include "cAlgBfish.h"
#include "Blowfish/Bc_bfish.h"

const char *cAlgBfish::getDisplayName()
{
	return DISPLAY_NAME;
}

cAlgBfish::cAlgBfish()
{
	m_extKey = NULL;
	m_keySize = 0;
}


cAlgBfish::~cAlgBfish()
{
	safeFree( m_extKey );
}


BOOL cAlgBfish::getAlgConstants( int *blockSize, int *keySize )
{
	*blockSize = BLOCK_SIZE;
	*keySize   = m_keySize;
	return TRUE;
}


int  cAlgBfish::getBlockSize()
{
	return BLOCK_SIZE;
}

int cAlgBfish::getKeySize( int *pMax )
{
	if (pMax) *pMax = MAX_KEY_LENGTH / 8;

	return PGP_DEFINED_KEY_LENGTH / 8;
}

int cAlgBfish448::getKeySize( int *pMax )
{
	if (pMax) *pMax = MAX_KEY_LENGTH / 8;

	return MAX_KEY_LENGTH / 8;
}

cSymAlg * cAlgBfish::createAlg( BYTE *key, int keySize ) // keySize in bytes
{
	BYTE *lKey;
	BOOL bMustFree = FALSE;

	cAlgBfish *alg = new cAlgBfish();
	
	if (keySize < (MIN_KEY_LENGTH / 8))
	{
		alg->m_keySize = MIN_KEY_LENGTH / 8;
		lKey = safeAlloc( MIN_KEY_LENGTH / 8 );
		if (!lKey) return NULL;
		memset( lKey, 0, MIN_KEY_LENGTH / 8 );
		memcpy( lKey, key, keySize );
		bMustFree = TRUE;
	}
	else
	{
		alg->m_keySize = (keySize > (MAX_KEY_LENGTH / 8)) ? (MAX_KEY_LENGTH / 8) : keySize;
		lKey = key;
	}

	alg->m_extKey = safeAlloc( KEY_LENGTH_EXTENDED / 8 );
	if (!alg->m_extKey)
	{
		if (bMustFree) safeFree( lKey );
		delete alg;
		return NULL;
	}

	if (!BFISH_KeyExtend( alg->m_keySize * 8, lKey, (UINT *)alg->m_extKey ))
	{
		if (bMustFree) safeFree( lKey );
		safeFree( alg->m_extKey );
		delete alg;
		return NULL;
	}

	if (bMustFree) safeFree( lKey );

	return alg;
}


/*  Macroses to transform array of 4 bytes to 32-bit words
 *  (and reverse) without depending on the Little-Endian or 
 *  Big-Endian processor's architecture 
 */
/*
#define BYTES_TO_UINT(b,d) (d =((UINT)(*((b)++))), \
                             d|=((UINT)(*((b)++)))<< 8, \
                             d|=((UINT)(*((b)++)))<<16, \
                             d|=((UINT)(*((b)++)))<<24)

#define UINT_TO_4BYTES(d,b) (*((b)++)=(BYTE)(((d)    )&0xff), \
                              *((b)++)=(BYTE)(((d)>> 8)&0xff), \
                              *((b)++)=(BYTE)(((d)>>16)&0xff), \
                              *((b)++)=(BYTE)(((d)>>24)&0xff))
*/

#define BYTES_TO_UINT(i,d) (d =(UINT)(inBf[i+3]), \
                             d|=(UINT)(inBf[i+2]) << 8, \
                             d|=(UINT)(inBf[i+1]) <<16, \
                             d|=(UINT)(inBf[i])   <<24)

#define UINT_TO_4BYTES(d,i) (outBf[i+3]=(BYTE)(((d)    )&0xff), \
                              outBf[i+2]=(BYTE)(((d)>> 8)&0xff), \
                              outBf[i+1]=(BYTE)(((d)>>16)&0xff), \
                              outBf[i]  =(BYTE)(((d)>>24)&0xff))



int cAlgBfish::encryptECB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int i;
	UINT left, right;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;
	
	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		BYTES_TO_UINT( i, left );
		BYTES_TO_UINT( i+sizeof(UINT), right );

		Blowfish_ECB_Encrypt( &left, &right, (BKEY *)m_extKey );

		UINT_TO_4BYTES( left,  i);
		UINT_TO_4BYTES( right, i+sizeof(UINT) );
	}

	return sae_ok;
}



int cAlgBfish::decryptECB(
		const BYTE *inBf,		// Input encrypted buffer.
		BYTE *outBf,	// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
{
	int i;

	UINT left, right;

	if ((bfSize % BLOCK_SIZE) != 0) return sae_invalidBlockSize;
	
	for( i = 0; i < bfSize; i += BLOCK_SIZE)
	{
		BYTES_TO_UINT( i, left );
		BYTES_TO_UINT( i+sizeof(UINT), right );

		Blowfish_ECB_Decrypt( &left, &right, (BKEY *)m_extKey );

		UINT_TO_4BYTES( left,  i);
		UINT_TO_4BYTES( right, i+sizeof(UINT) );
	}

	return sae_ok;
}
