#include <string.h>
#include "cSymAlg.h"


cSymAlg::cSymAlg()
{
	m_iv = NULL;
	m_prev = NULL;
	m_iv_offset = 0;
}


cSymAlg::~cSymAlg()
{
	if (m_iv != NULL) safeFree( m_iv );
	m_iv = NULL;
	
	if (m_prev != NULL) safeFree( m_prev );
	m_prev = NULL;

	m_iv_offset = 0;
}



// Encryption and decription in Cipher Feedback mode

int cSymAlg::initCFB( const BYTE *iv ) // iv - Initial vector
{
	int blockSize, keySize;
	
//	if ( iv == NULL ) return sae_invalidBlockSize;

	if (!getAlgConstants( &blockSize, &keySize )) return sae_internalError;

	if (!m_iv) m_iv = safeAlloc( blockSize );
	if (!m_iv) return sae_memoryAllocationError; // sae_internalError;

	if (!m_prev) m_prev = safeAlloc( blockSize );
	if (!m_prev)
	{
		safeFree( m_iv );
		return sae_memoryAllocationError; // sae_internalError;
	}

	if ( iv == NULL )
		memset( m_iv, 0, blockSize );
	else
		memcpy( m_iv, iv, blockSize );

	m_iv_offset = 0;

	return sae_ok;
}



int cSymAlg::encryptCFB(
		const BYTE *inBf,		// Input plaintext buffer. 
		BYTE *outBf,	// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
{
	int blockSize, keySize, i, iRes;
		
	if (!getAlgConstants( &blockSize, &keySize )) return sae_internalError;

	for (i=0; i<bfSize; i++, m_iv_offset++ )
	{
		m_iv_offset %= blockSize;

		if (m_iv_offset == 0)
		{
			memcpy( m_prev, m_iv, blockSize );
			iRes = encryptECB( m_iv, m_iv, blockSize );
			if (iRes != sae_ok) return iRes;
		}

		outBf[i] = inBf[i] ^ m_iv[m_iv_offset];
		m_iv[m_iv_offset] = outBf[i];
	}

	return sae_ok;
}


int cSymAlg::decryptCFB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize			// In/out buffers' size.
		)
{
	int  blockSize, keySize, i, iRes;
	BYTE bTmp;
		
	if (!getAlgConstants( &blockSize, &keySize )) return sae_internalError;

	for (i=0; i<bfSize; i++, m_iv_offset++ )
	{
		m_iv_offset %= blockSize;

		if (m_iv_offset == 0)
		{
			memcpy( m_prev, m_iv, blockSize );
			iRes = encryptECB( m_iv, m_iv, blockSize );
			if (iRes != sae_ok) return iRes;
		}

		bTmp = inBf[i]; // We need bTmp because inBf may be the same as outBf
		outBf[i] = m_iv[m_iv_offset] ^ inBf[i];
		m_iv[m_iv_offset] = bTmp;
	}

	return sae_ok;
}
/*
	static void
pgpCFBSync(CFBInterleaveStruct *cfb)
{
	PGPSize	blockSize;
	PGPSize	bufLeft		= cfb->bufLeft;

	(void)PGPGetSymmetricCipherSizes( cfb->symmetricRef, NULL, &blockSize );
	
	if (bufLeft)
	{
		pgpCopyMemory( cfb->iv, cfb->iv + bufLeft, blockSize - bufLeft);
		pgpCopyMemory( cfb->prev + blockSize - bufLeft, cfb->iv, bufLeft);
		cfb->bufLeft	= 0;
	}
}
*/

int cSymAlg::syncCFB()
{
	int  blockSize, keySize, bytesLeft;
		
	if (!getAlgConstants( &blockSize, &keySize )) return sae_internalError;

	bytesLeft = blockSize - m_iv_offset;

	if ( bytesLeft )
	{
		memmove( m_iv, &(m_iv[ bytesLeft ]), m_iv_offset );
		memcpy( &(m_prev[m_iv_offset]), m_iv, bytesLeft );
		m_iv_offset = blockSize;
	}

	return sae_ok;
}

int cSymAlg::encryptCBC(
	const BYTE *sync,     // Initial vector
	const BYTE *inBf,		// Input plaintext buffer. 
	BYTE *outBf,	// Output encrypted buffer
	int  bfSize		// In/out buffers' size.
	)
{
	int   blockSize, keySize, uintsNumber, i, j;
	UINT internalBuf[16];
	
	if (!getAlgConstants( &blockSize, &keySize )) return sae_internalError;
	
	if (!blockSize) return sae_invalidBlockSize;
	if ((bfSize % blockSize) != 0) return sae_invalidBlockSize;

	uintsNumber = blockSize / sizeof(UINT);

	for (i=0; i<uintsNumber; i++)
		internalBuf[i] = ((UINT *)sync)[i];

	for (i=0; i < (bfSize / (int)sizeof(UINT)); i=i+uintsNumber )
	{
		// do EBC encryption of (Initial_Vector XOR Data)
		for (j=0; j<uintsNumber; j++) internalBuf[j] ^= ((UINT *)inBf)[i+j];

		if (encryptECB( (BYTE *)internalBuf, (BYTE *)internalBuf, blockSize ) != sae_ok )
			return sae_operationError;

		for (j=0; j<uintsNumber; j++) ((UINT *)outBf)[i+j] = internalBuf[j];
	}

	return sae_ok;
}

//
// Note, The function is modified 7 Oct 2008.
// Solaris code cannot properly copy UINT buffer if it's boundary is not divided by 4
// So the places have been changed to memcpy here.
//
int cSymAlg::decryptCBC(
	const BYTE *sync,     // Initial vector
	const BYTE *inBf,		// Input encrypted buffer.
	BYTE *outBf,	// Output plaintext buffer.
	int  bfSize		// In/out buffers' size.
	)
{
	int   blockSize, keySize, uintsNumber, i, j;
	UINT internalBuf[16], oldBuf[16], ivector[16];
	
	if (!getAlgConstants( &blockSize, &keySize )) return sae_internalError;
	
	if (!blockSize) return sae_invalidBlockSize;
	if ((bfSize % blockSize) != 0) return sae_invalidBlockSize;

	uintsNumber = blockSize / sizeof(UINT);

	//See Note above, for (i=0; i<uintsNumber; i++) ivector[i] = ((UINT *)sync)[i];
	memcpy( ivector, sync, blockSize );

	for ( i=0; i < (bfSize / (int)sizeof(UINT)); i += uintsNumber )
	{
		//See Note above,for (j=0; j<uintsNumber; j++) internalBuf[j] = oldBuf[j] = ((UINT *)inBf)[i+j];
		memcpy( internalBuf, (UINT*)inBf + i, blockSize );
		memcpy( oldBuf, internalBuf, blockSize );

		// Encrypted Data -> new IV, 
		// then do EBC decryption of Encrypted Data, 
		// then XOR decrypted data with old IV
		if (decryptECB( (BYTE *)internalBuf, (BYTE *)internalBuf, blockSize ) != sae_ok )
			return sae_operationError;

		//See Note above, for (j=0; j<uintsNumber; j++) ((UINT *)outBf)[i+j] = internalBuf[j] ^ ivector[j];
		for (j=0; j<uintsNumber; j++) internalBuf[j] ^= ivector[j];
		memcpy( (UINT*)outBf + i, internalBuf, blockSize );

		for (j=0; j<uintsNumber; j++) ivector[j] = oldBuf[j]; 

	}

	return sae_ok;
}
