/////////////////////////////////////////////////////////////////
//
//      Copyright (c) 1994-2008 Jetico, Inc., Finland
//      All rights reserved.
//
//      File: cSymAlg.h
//      Revision:
//      Created:
//      Description: Base class for all symmetric algorithms
//
/////////////////////////////////////////////////////////////////

#ifndef __cSymAlg__
#define __cSymAlg__

#include <stdlib.h>
#include <memory.h>
#include "../SMemory/smemory.h"
#include "ALG_DEF.H"

#include "../crtLibErrs.h"

class cSymAlg
{
  private:

	BYTE *m_iv;
	BYTE *m_prev;
	int  m_iv_offset;
  
  protected:

	static BYTE *safeAlloc( int size ) 	{ return (BYTE *)SMalloc( size ); }
	static void safeFree( BYTE *mem ) 	{ SFree( mem ); }


  public:

	cSymAlg();
	~cSymAlg();
#ifdef _WIN32
#pragma warning( disable : 4100 )
#endif

	virtual BOOL getAlgConstants( int *blockSize, int *keySize ) { return FALSE; }

	// Encryption and decription in Electronic Code Book mode has to be implemented at child class

	virtual int encryptECB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		)
	{ return sae_internalError; }

	virtual int decryptECB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		)
	{ return sae_internalError; }

#ifdef _WIN32
#pragma warning( default : 4100 )
#endif
	//
	//	Encryption and decription in Cipher Block Chaining mode
	//		Both function return sae_invalidBlockSize if buffer size 
	//		is not devided by algorithm block size.
	//

	int encryptCBC(
		const BYTE *sync,	// Initial vector
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		);
	int decryptCBC(
		const BYTE *sync,	// Initial vector
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		);

	//
	//	Encryption and decription in Cipher Feedback mode
	//		
	int initCFB( const BYTE *iv );
	int syncCFB(); // see the NOTE below

	int encryptCFB(
		const BYTE *inBf,	// Input plaintext buffer. 
		BYTE *outBf,		// Output encrypted buffer
		int  bfSize		// In/out buffers' size.
		);

	int decryptCFB(
		const BYTE *inBf,	// Input encrypted buffer.
		BYTE *outBf,		// Output plaintext buffer.
		int  bfSize		// In/out buffers' size.
		);
};


#endif // __cSymAlg__


/************* NOTE concerning syncCFB  ************************************

From:
Network Working Group
Request for Comments: 2440
Category: Standards Track
(RFC2440)

   The data is encrypted in CFB mode, with a CFB shift size equal to the
   cipher's block size.  The Initial Vector (IV) is specified as all
   zeros.  Instead of using an IV, OpenPGP prefixes a 10-octet string to
   the data before it is encrypted.  The first eight octets are random,
   and the 9th and 10th octets are copies of the 7th and 8th octets,
   respectively. After encrypting the first 10 octets, the CFB state is
   resynchronized if the cipher block size is 8 octets or less.  The
   last 8 octets of ciphertext are passed through the cipher and the
   block boundary is reset.

****************************************************************************/
