/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 */

#ifndef BCBUS_CRYPTO_MEMORY_H
#define BCBUS_CRYPTO_MEMORY_H

#include <IOKit/IOLib.h>
#include "types.h"

/**
 * Allocate a block of kernel memory.
 * @p size	Memory size.
 * @return	Memory buffer pointer or NULL if allocation failed.
 */
static inline void* bc_alloc(size_t size)
{
	void* mem = IOMalloc(size);
	if(mem) {
		memset(mem, 0, size);
	}
	
	return mem;
}


/**
 * Overwrite and free a block memory.
 * @p mem	Pointer to allocated memory.
 * @p size	Size of the memory buffer. Need for overwriting.
 */
static inline void bc_shred(void* mem, size_t size)
{
	memset(mem, 0, size);
	IOFree(mem, size);
}


#endif
