/*
 * Copyright 2010 Jetico, Inc. All rights reserved.
 *
 * SHA1/SHA256 interface.
 * We have an internal id for both supported hash algorithms.
 * Currently the implementation just piggybacks on CertificateLib and we basically map our interface to it.
 */

#ifndef KGSHA_SHA_H
#define KGSHA_SHA_H

#include "defs.h"
#include <stdlib.h>

// Hash algorithm internal id.
// This is what we get from container header and map to CertificateLib.
enum {
	kSHA_ID			= 0x53484131,
	kSHA256_ID		= 0x53323536,
};


/// Get digest size for sha version.
size_t KGSHA_DigestSize(int hash_id);


/// Calculate message digest.
void KGSHA_CalculateDigest(int hash_id, const char *data, size_t length, char *out_digest);


/// Generate random data with hash algorithm.
void KGSHA_RandomGenerate(int hash_id, const char* seed, size_t seed_size, char* out_buf, size_t nbytes);


/// Overwrite a memory area with sha-generated garbage.
void KGSHA_ShredData(char *data, size_t nbytes);


/// Generate random encryption key data using sha hash.
void KGSHA_RandomGenerateKey(int hash_id, const char* seed, size_t seed_length, char *out_key, size_t out_key_size);


#endif
